/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * JBIListener.java - Initial Developper : ofabre
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.se.xslt.listener;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.logging.Level;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.jbi.messaging.MessagingException;
import javax.mail.util.ByteArrayDataSource;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.TransformerException;

import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.StringHelper;
import org.ow2.petals.se.xslt.Constants;
import org.ow2.petals.se.xslt.XsltComponent;
import org.ow2.petals.se.xslt.service.XsltService;

/**
 * The XSLT binding component JBIListener.
 * 
 * @author alouis, ofabre, cgirodengo
 * 
 */
public class JBIListener extends AbstractJBIListener {

    private XsltService xsltService;

    /**
     * Validate the operation and the MEP.
     * 
     * @param operation
     */
    private static final boolean validateExchange(final Exchange exchange) {
        boolean result = true;

        final QName operation = exchange.getOperation();
        if (!operation.getNamespaceURI().equals(Constants.XSLT_NS)) {
            exchange.setError(new Exception("Operation '" + operation
                    + "' must be in the namespace '" + Constants.XSLT_NS + "'"));
            result = false;
        }

        if (Constants.OPERATION_TRANSFORM.equals(operation.getLocalPart().toLowerCase())) {
            if (!Message.MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern())) {
                exchange.setError(new Exception("Operation '" + operation
                        + "' must use the pattern '" + Message.MEPConstants.IN_OUT_PATTERN.value()
                        + "'"));
                result = false;
            }
            return result;
        } else if (Constants.OPERATION_TRANSFORM_TO_ATTACHMENT.equals(operation.getLocalPart()
                .toLowerCase())) {
            // TODO implement me
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.petals.component.framework.listener.AbstractListener#init()
     */
    @Override
    public void init() {
        this.xsltService = new XsltService();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.petals.component.framework.listener.AbstractJBIListener
     * #onJBIMessage(org.objectweb.petals.component.framework.util.Exchange)
     */
    @Override
    public boolean onJBIMessage(final Exchange exchange) {

        if (!validateExchange(exchange)) {
            this.getLogger().log(Level.WARNING, exchange.getError().getMessage());
            // no SQL request, error is set in the exchange
            return true;
        }

        try {
            this.processExchange(exchange);
        } catch (final Exception e) {
            this.getLogger().log(Level.WARNING,
                    e.getClass().getSimpleName() + " : " + e.getMessage());
            // technical error, it would be set as a Fault by the CDK
            if (!exchange.isActiveStatus()) {
                exchange.setError(new Exception(e));
            }
        }

        return true;
    }

    /**
     * Transform the outputstream as an IN DataHandler
     * 
     * @param outputStream
     * @param attachmentName
     * @return
     * @throws IOException
     */
    private DataHandler createDataSourceFromOutputStream(final ByteArrayOutputStream outputStream)
    throws IOException {
        this.getLogger().log(Level.INFO, "Put the result as an attachment");
        final ByteArrayInputStream input = new ByteArrayInputStream(outputStream.toByteArray());
        final DataSource datasource = new ByteArrayDataSource(input, "application/octet-stream");
        return new DataHandler(datasource);
    }

    /**
     * Process the transformation and set the response as Source or attachment
     * 
     * @param exchange
     * @throws MessagingException
     * @throws BusinessException
     * @throws MessagingException
     * @throws TransformerException
     * @throws IOException
     */
    private void processExchange(final Exchange exchange) throws MessagingException,
    TransformerException, IOException {
        this.getLogger().log(Level.INFO, "Process the transformation");

        //
        // TODO: the attachment mode must be handled in a separate operation!
        //

        // Get in Message content
        final Source inContent = exchange.getInMessageContentAsSource();

        // get the XSL template for current Service Unit
        final Templates xslTransformers = ((XsltComponent) this.component).getTemplates().get(
                exchange.getEndpointName());

        final ClassLoader customClassLoader = ((XsltComponent) this.component)
        .getCustomizedClassLoader().get(exchange.getEndpointName());
        if (customClassLoader != null) {
            Thread.currentThread().setContextClassLoader(customClassLoader);
        }

        // transform the input
        final ByteArrayOutputStream outContent = this.xsltService.transform(inContent,
                xslTransformers, this.getInstallRoot());

        if (customClassLoader != null) {
            Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());
        }

        final String outputAttachmentName = this.getExtensions().get(
                Constants.OUTPUT_ATTACHMENT_NAME);
        if (StringHelper.isNullOrEmpty(outputAttachmentName)) {
            exchange.setOutMessageContent(new ByteArrayInputStream(outContent.toByteArray()));
        } else {
            // set the response as attachment
            final DataHandler attachment = this.createDataSourceFromOutputStream(outContent);
            exchange.setOutMessageAttachment(outputAttachmentName, attachment);
            final String outSource = "<?xml version=\"1.0\" encoding=\"UTF-8\"?><attached-files><file-name>"
                + outputAttachmentName + "</file-name></attached-files>";
            exchange.setOutMessageContent(new ByteArrayInputStream(outSource.getBytes("UTF-8")));
        }
    }
}
